unit ReportU;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, Db, DBTables;

type
  TReport = class(TForm)
    ReportPanel: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    FromLabel: TLabel;
    ToLabel: TLabel;
    DateLabel: TLabel;
    Memo: TMemo;
    JobButton: TButton;
    EmployeeButton: TButton;
    NextButton: TButton;
    Query: TQuery;
    AcceptButton: TButton;
    RejectButton: TButton;
    procedure FormShow(Sender: TObject);
    procedure JobButtonClick(Sender: TObject);
    procedure EmployeeButtonClick(Sender: TObject);
    procedure NextButtonClick(Sender: TObject);
    procedure RejectButtonClick(Sender: TObject);
    procedure AcceptButtonClick(Sender: TObject);
  private
    { Private declarations }
    EmployeeID: Integer;
    EmployerID: Integer;
    JobID: Integer;
    ReportID: Integer;
    From: string;
    FromS: string;
    Job: string;
    ToS: string;
    ThirdS: string;
    Accept: Boolean;
    procedure ReadReport;
    procedure OpenQuery;
  public
    { Public declarations }
    procedure SendReport(EID, YID, JID: Integer);
    procedure SetReportText;
  end;

var
  Report: TReport;

implementation

{$R *.DFM}

uses
  Global, EmployersListU, EmployeesListU;

procedure TReport.SetReportText;
begin
  AcceptButton.Visible := False;
  RejectButton.Visible := False;
  NextButton.Enabled := True;
  showContact := False;
  if Global.WhoAmI = 'SysAdmin' then
  begin
    AcceptButton.Visible := True;
    RejectButton.Visible := True;
    NextButton.Enabled := False;
    if From = 'Employee' then
      Memo.Text := FromS+' has accepted job '+Job+' from '+ToS+'. '
                  +'You can accept it, that will remove the employee and one job from the system, or reject it.'
    else
      Memo.Text := FromS+' has accepted '+ToS+' for job '+Job+'. '
                  +'You can accept it, that will remove the employee and one job from the system, or reject it.'
  end
  else if Global.WhoAmI = 'Employee' then
  begin
    if From = 'SysAdmin' then
      if Accept then
      begin
        Memo.Text := FromS+' has accepted the job '+Job+' of '+ThirdS+' for you. '
                    +'Please remember the contact information of the employer '
                    +'that can be found from the Job Info button. '
                    +'Your account will be removed from system, '
                    +'and you cannot login with your current UserID anymore.';
        showContact := true;
      end
      else
        Memo.Text := FromS+' has rejected the job '+Job+' of '+ThirdS+' for you.'
    else
      Memo.Text := FromS+' has accepted you for job '+Job+'. '
                  +'You should wait for the System Operator to accept too.'
  end
  else if Global.WhoAmI = 'Employer' then
  begin
    if From = 'SysAdmin' then
    begin
      if Accept then
      begin
        Memo.Text := FromS+' has accepted '+ThirdS+' for your job '+Job+'. '
                    +'Please remember the contact information of the employee '
                    +'that can be found from the Employee Info button. '
                    +'One of your job requests will be removed from the system, '
                    +'and if you have no more requests, your UserID will be removed too.';
        showContact := true;
      end
      else
        Memo.Text := FromS+' has rejected the request of '+ThirdS+' for your job '+Job+'.';
    end
    else
      Memo.Text := FromS+' has accepted you for job '+Job+'. '
                  +'You should wait for the System Operator to accept too.'
  end;
end;

procedure TReport.ReadReport;
begin
  if (NextButton.Caption = 'Close') or Query.EOF then
    Close;
  if ReportID <> -1 then
  begin
    Query.Close;
    Query.SQL.Clear;
    Query.SQL.Add('Delete from Report where ReportID='+IntToStr(ReportID));
    Query.ExecSQL;
    OpenQuery;
    Query.Next;
  end;
  ReportID := Query.Fields.FieldByName('ReportID').AsInteger;
  if Query.EOF then
  begin
    NextButton.Caption := 'Close';
    Close;
    Exit;
  end;
  EmployeeID := Query.Fields.FieldByName('EmployeeID').AsInteger;
  EmployerID := Query.Fields.FieldByName('EmployerID').AsInteger;
  JobID := Query.Fields.FieldByName('JobID').AsInteger;
  From := Query.Fields.FieldByName('From').AsString;
  Job := Query.Fields.FieldByName('Job').AsString;
  Accept := Query.Fields.FieldByName('Accept').AsBoolean;
  if From = 'SysAdmin' then
  begin
    FromS := 'System Operator';
    if WhoAmI = 'Employee' then
    begin
      ToS   := Query.Fields.FieldByName('First Name').AsString+' '+
               Query.Fields.FieldByName('Last Name').AsString;
      ThirdS:= Query.Fields.FieldByName('Title').AsString;
    end
    else
    begin
      ToS   := Query.Fields.FieldByName('Title').AsString;
      ThirdS:= Query.Fields.FieldByName('First Name').AsString+' '+
               Query.Fields.FieldByName('Last Name').AsString
    end;
  end
  else if From = 'Employer' then
  begin
    FromS := Query.Fields.FieldByName('Title').AsString;
    ToS   := Query.Fields.FieldByName('First Name').AsString+' '+
             Query.Fields.FieldByName('Last Name').AsString;
  end
  else
  begin
    FromS := Query.Fields.FieldByName('First Name').AsString+' '+
             Query.Fields.FieldByName('Last Name').AsString;
    ToS   := Query.Fields.FieldByName('Title').AsString;
  end;
  FromLabel.Caption := FromS;
  ToLabel.Caption   := ToS;
  DateLabel.Caption := Query.Fields.FieldByName('Date').AsString;
  SetReportText;
end;

procedure TReport.OpenQuery;
begin
  with Query.SQL do
  begin
    Clear;
    Add('select * from Report R, Employee E, Employer Y, Job J '
        +'where R.EmployeeID=E.ID and R.EmployerID=Y.ID and R.JobID=J.ID '
        +' and R."For"="'+Global.WhoAmI+'" order by ReportID ');
  end;
  NextButton.Caption := 'Delete this,  Next >>';
  Memo.Text := '';
  Query.Active := True;
  if Query.Eof then
    NextButton.Caption := 'Close';
  ReportID := -1;
end;

procedure TReport.FormShow(Sender: TObject);
begin
  Global.ResetVars;
  Global.isReport := True;
  OpenQuery;
  ReadReport;
end;

procedure TReport.JobButtonClick(Sender: TObject);
begin
  Global.JobFilter := 'ID='+IntToStr(JobID);
  Global.JobSQL := 'select * from Job, Degree where Degree.ID=Job.Degree;';
  Global.isSearch := True;
  EmployersList.ShowModal;
  Global.isSearch := False;
  Global.JobFilter := '';
  Global.JobSQL := '';
end;

procedure TReport.EmployeeButtonClick(Sender: TObject);
begin
  Global.Filter := 'ID='+IntToStr(EmployeeID);
  Global.SQL := 'select * from Employee;';
  Global.isSearch := True;
  EmployeesList.ShowModal;
  Global.isSearch := False;
  Global.Filter := '';
  Global.SQL := '';
end;

procedure TReport.SendReport(EID, YID, JID: Integer);
const
  Names: array [0 .. 2] of string = ('SysAdmin', 'Employee', 'Employer');
var
  Ss: string;
  Forx: string;
  I: Integer;
begin
  DateSeparator := '-';
  for I := 0 to 2 do
  begin
    Forx := Names[I];
    if WhoAmI = Forx then
      Continue;
    with Query.SQL do
    begin
      Clear;
      Ss := ('insert into report (EmployeeID, EmployerID, JobID, Report."From", Accept, Report."Date", Report."For") '
          +'values ('+IntToStr(EID)+','+IntToStr(YID)+','+IntToStr(JID)+',"'
          +WhoAmI+'",'+B(Global.Accept)+','+S(DateToStr(Date))+','+S(Forx)+');');
      Add(Ss);
    end;
    Query.ExecSQL;
  end;
  if WhoAmI <> 'SysAdmin' then
    MessageDlg('A request has been sent to System Operator, the result will be sent to you as a report',
               mtInformation, [mbOK], 0)
  else
    MessageDlg('Reports has been sent to both employer and employee',
               mtInformation, [mbOK], 0);
end;

procedure TReport.NextButtonClick(Sender: TObject);
begin
  ReadReport;
end;

procedure TReport.RejectButtonClick(Sender: TObject);
begin
  Global.Accept := false;
  Query.Close;
  SendReport(EmployeeID, EmployerID, JobID);
  OpenQuery;
  ReportID := Query.Fields.FieldByName('ReportID').AsInteger;
  ReadReport;
end;

procedure TReport.AcceptButtonClick(Sender: TObject);
begin
  Query.Close;
  with Query do
  begin
    SQL.Clear;
    SQL.Add('');
    SQL[0] := 'delete from Report where EmployeeID='+IntToStr(EmployeeID)+';';
    ExecSQL;
    SQL[0] := 'delete from Employee_City where EmployeeID='+IntToStr(EmployeeID)+';';
    ExecSQL;
    SQL[0] := 'delete from Employee_Degree where EmployeeID='+IntToStr(EmployeeID)+';';
    ExecSQL;
    SQL[0] := 'update Employee set Found="true" where ID='+IntToStr(EmployeeID)+';';
    ExecSQL;
    SQL[0] := 'update Job set Requests=Requests-1, Filled=Filled+1 where ID='+IntToStr(JobID)+';';
    ExecSQL;
    SQL[0] := 'select * from Job where ID='+IntToStr(JobID)+';';
    Active := True;
    if Fields.FieldByName('Requests').AsInteger <= 0 then
    begin
      Close;
      SQL[0] := 'delete from Report where JobID='+IntToStr(JobID)+';';
      ExecSQL;
    end
    else
      Close;
  end;
  Global.Accept := true;
  SendReport(EmployeeID, EmployerID, JobID);
  Global.Accept := false;
  OpenQuery;
  ReportID := Query.Fields.FieldByName('ReportID').AsInteger;
  ReadReport;
end;

end.

